{% extends "project/databases/_detail_overview.html" %}
{% load i18n sizeformat %}

{% block connection_info %}
    <h4>{% trans "Connection Information" %}</h4>
    <hr class="header_rule">
    <dl class="dl-horizontal">
    {% if instance.cluster_id %}
        <a href="/project/clusters/{{instance.cluster_id}}">Link</a> to Cluster Details for Connection Information
    {% else %}
        <dl>
        {% with instance.host as host %}
            <dt>{% trans "Host" %}</dt>
            <dd>
            {% if not host %}
                {% trans "Not Assigned" %}
            </dd>
            {% else %}
                {{ host }}
            </dd>
            <dt>{% trans "Database Port" %}</dt>
            <dd>27017</dd>
            <dt>{% trans "Connection Examples" %}</dt>
            <dd>mongo --host {{ host }}</dd>
            <dd>mongodb://[{% trans "USERNAME" %}:{% trans "PASSWORD" %}@]{{ host }}:27017/{% trans "DATABASE" %}</dd>
            {% endif %} <!-- ends else block -->
        {% endwith %}
        </dl>
    {% endif %}
{% endblock %}
