{% load i18n %}

<br>
<div class="help_text">
  {% trans "Add parameters to the configuration group.  When all the parameters are added click 'Apply Changes' to persist changes." %}
</div>
<br>
{{ table.render }}
