{% extends "base.html" %}
{% load i18n %}
{% block title %}{% trans "Reset Root Password" %}{% endblock %}

{% block main %}
  {% include "project/database_clusters/_reset_password.html" %}
{% endblock %}